/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui.preference;

import java.io.File;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import benten.cat.glossary.ui.CatGlossaryUiPlugin;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.fields.AbstractBrowseTextField;
import benten.ui.fields.FolderTextField;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;
import benten.ui.views.UiToolkit;

/**
 * 用語集設定ページ。
 * 
 * @author IGA Tosiki
 */
public class CatGlossaryUiPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private final UiToolkit toolkit = new UiToolkit();
	private Combo glossaryNameCombo;
	private Button useTsvDefaultCheck;
	private Button useTsvLocationCheck;
	private AbstractBrowseTextField tsvLocationText;

	public CatGlossaryUiPreferencePage() {
	}

	public CatGlossaryUiPreferencePage(String title) {
		super(title);
	}

	public CatGlossaryUiPreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		final Composite composite = toolkit.createComposite(parent, 1);
		final IPreferenceStore store = BentenPreference.getStore();

		final Composite c = toolkit.createComposite(composite, 2);
		c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		final Set<String> glossaryNameSet = CatGlossaryUiPlugin.getDefault().getTmNameSet();
		toolkit.createLabel(c, "用語集:");
		glossaryNameCombo = new Combo(c, SWT.READ_ONLY);
		glossaryNameCombo.setItems(glossaryNameSet.toArray(new String[glossaryNameSet.size()]));
		glossaryNameCombo.setText(store.getString(Preference.GLOSSARY_NAME.name()));

		createTsvGroup(composite);

		return composite;
	}

	protected void createTsvGroup(Composite parent) {
		final IPreferenceStore store = BentenPreference.getStore();

		final Group g = new Group(parent, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(4, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText("使用する用語集データ(TSV)のロケーション");

		final GridData gd = new GridData();
		gd.horizontalSpan = 4;
		useTsvDefaultCheck = new Button(g, SWT.RADIO);
		useTsvDefaultCheck.setText("プラグイン組み込み用語集データ(TSV)");
		useTsvDefaultCheck.setSelection(store.getBoolean(Preference.USE_TSV_DEFAULT.name()));
		useTsvDefaultCheck.setLayoutData(gd);
		useTsvDefaultCheck.addSelectionListener(new TsvSelectionAdapter());

		useTsvLocationCheck = new Button(g, SWT.RADIO);
		useTsvLocationCheck.setText("次のロケーション:");
		useTsvLocationCheck.setSelection(store.getBoolean(Preference.USE_TSV_LOCATION.name()));
		useTsvLocationCheck.addSelectionListener(new TsvSelectionAdapter());

		tsvLocationText = new FolderTextField(g);
		tsvLocationText.setWidthHint(SWT.DEFAULT);
		tsvLocationText.setText(store.getString(Preference.TSV_LOCATION.name()));

		if (!useTsvDefaultCheck.getSelection() && !useTsvLocationCheck.getSelection()) {
			useTsvDefaultCheck.setSelection(true);
		}
		updateTsvCheck();
	}

	private class TsvSelectionAdapter extends SelectionAdapter {
		@Override
		public void widgetSelected(SelectionEvent e) {
			updateTsvCheck();
		}
	}

	protected void updateTsvCheck() {
		tsvLocationText.setEnabled(useTsvLocationCheck.getSelection());
	}

	/**
	 * {@inheritDoc}
	 */
	public void init(IWorkbench workbench) {
	}

	@Override
	public boolean performOk() {
		final IPreferenceStore store = BentenPreference.getStore();

		final String newGlossaryName = glossaryNameCombo.getText();
		final boolean newUseTsvDefault = useTsvDefaultCheck.getSelection();
		final boolean newUseTsvLocation = useTsvLocationCheck.getSelection();

		// TSV ロケーションのチェック
		if (newUseTsvLocation) {
			if (tsvLocationText.getText().equals("")) {
				final UiStatus s = new UiStatus(IStatus.ERROR, "用語集データ・ロケーションを指定してください。");
				UiPlugin.openDialog("入力エラー", s);
				return false;
			}
			final File tsvDir = tsvLocationText.getFile();
			if (!tsvDir.exists() || tsvDir.isFile()) {
				final UiStatus s = new UiStatus(IStatus.ERROR, "指定された用語集データ・ディレクトリーは存在しません。");
				UiPlugin.openDialog("入力エラー", s);
				return false;
			}
		}

		final String oldGlossaryName = store.getString(Preference.GLOSSARY_NAME.name());
		final boolean oldUseTsvDefault = store.getBoolean(Preference.USE_TSV_DEFAULT.name());
		final boolean oldUseTsvLocation = store.getBoolean(Preference.USE_TSV_LOCATION.name());
		final String oldTsvLocation = store.getString(Preference.TSV_LOCATION.name());

		// 設定が load メソッド内で参照されるため、先に保管
		store.setValue(Preference.USE_TSV_DEFAULT.name(), useTsvDefaultCheck.getSelection());
		store.setValue(Preference.USE_TSV_LOCATION.name(), useTsvLocationCheck.getSelection());
		store.setValue(Preference.TSV_LOCATION.name(), tsvLocationText.getText());

		if (!newGlossaryName.equals(oldGlossaryName)) {

			// 用語集エンジン・プラグイン名が変更された場合は拡張プラグインをロードしなおす
			store.setValue(Preference.GLOSSARY_NAME.name(), newGlossaryName);
			CatGlossaryUiPlugin.getDefault().loadExtensions();

		} else {

			// 使用する用語集エンジン・プラグインが変更された場合は用語集データをロードしなおす
			if (newUseTsvDefault != oldUseTsvDefault || newUseTsvLocation) {
				try {
					CatGlossaryUiPlugin.getDefault().getGlossaryEnginePlugin().load();

				} catch (final Exception e) {

					// エラーの場合は設定を戻す
					useTsvDefaultCheck.setSelection(oldUseTsvDefault);
					useTsvLocationCheck.setSelection(oldUseTsvLocation);
					tsvLocationText.setText(oldTsvLocation);
					final UiStatus s = new UiStatus(IStatus.ERROR, e.getMessage() + "\n有効な用語集データ設定を行ってください。");
					UiPlugin.openDialog("用語集データのロードに失敗しました", s);
					return false;
				}
			}
		}

		return true;
	}

	@Override
	protected void performDefaults() {
		final IPreferenceStore store = BentenPreference.getStore();
		glossaryNameCombo.setText(store.getDefaultString(Preference.GLOSSARY_NAME.name()));

		useTsvDefaultCheck.setSelection(store.getDefaultBoolean(Preference.USE_TSV_DEFAULT.name()));
		useTsvLocationCheck.setSelection(store.getDefaultBoolean(Preference.USE_TSV_LOCATION.name()));
		tsvLocationText.setText(store.getDefaultString(Preference.TSV_LOCATION.name()));
	}
}
