/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.core.BentenGlossarySearchResult;
import benten.cat.glossary.ui.CatGlossaryUiPlugin;
import benten.cat.glossary.ui.actions.CopyGlossarySourceAction;
import benten.cat.glossary.ui.actions.CopyGlossaryTargetAction;
import benten.ui.views.CopySupportTableView;

/**
 * 用語集ビューです。
 * 
 * ★基本設計「翻訳支援機能: 用語対訳集検索機能」に対応します。
 */
public class GlossaryView extends CopySupportTableView {

	private Action copySourceAction;
	private Action copyTargetAction;
	private Action reloadAction;

	private TableColumn source;
	private TableColumn locColumn;
	private TableColumn comment;

	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {

		public String getColumnText(Object obj, int index) {
			final BentenGlossarySearchResult entry = (BentenGlossarySearchResult) obj;
			switch (index) {
			case 0:
				return entry.getSource();
			case 1:
				return entry.getTarget();
			case 2:
				return entry.getNote();
			}
			return getText(obj);
		}

		public Image getColumnImage(Object obj, int index) {
			return null;
		}
	}

	class NameSorter extends ViewerSorter {
		@Override
		public int compare(Viewer view, Object obj1, Object obj2) {
			return ((BentenGlossarySearchResult) obj1).getSource().compareToIgnoreCase(
					((BentenGlossarySearchResult) obj2).getSource());
		}
	}

	/**
	 * コンストラクター。.
	 */
	public GlossaryView() {
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 *
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		source = new TableColumn(table, SWT.LEFT);
		source.setText("ソース");
		source.setWidth(100);

		locColumn = new TableColumn(table, SWT.LEFT);
		locColumn.setText("ターゲット");
		locColumn.setWidth(100);

		comment = new TableColumn(table, SWT.LEFT);
		comment.setText("ノート");
		comment.setWidth(100);

		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(getItems());

		makeActions();
		createContextMenu();
		contributeToActionBars();
	}

	private List<BentenGlossarySearchResult> getItems() {
		return new ArrayList<BentenGlossarySearchResult>();
	}

	private void createContextMenu() {
		final MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				GlossaryView.this.fillContextMenu(manager);
			}
		});
		final Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		final IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(reloadAction);
		manager.add(new Separator());
	}

	private void fillContextMenu(IMenuManager manager) {
		final boolean isEmpty = viewer.getSelection().isEmpty();
		copySourceAction.setEnabled(!isEmpty);
		manager.add(copySourceAction);
		copyTargetAction.setEnabled(!isEmpty);
		manager.add(copyTargetAction);

		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(reloadAction);
	}

	private void makeActions() {
		reloadAction = new Action() {
			@Override
			public void run() {
				CatGlossaryUiPlugin.getDefault().getGlossaryEnginePlugin().load();
			}
		};
		reloadAction.setText("用語集のリロード");
		reloadAction.setToolTipText("用語集のリロード");
		reloadAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
				ISharedImages.IMG_OBJS_INFO_TSK));

		copySourceAction = new CopyGlossarySourceAction(this, "ソースのコピー");
		copyTargetAction = new CopyGlossaryTargetAction(this, "ターゲットのコピー");
	}

	public void showGlossary(String string) {
		final StringBuffer targetStrBuf = new StringBuffer();
		if (string != null) {
			targetStrBuf.append(string);
		}
		final BentenGlossaryEngine glossaryEngine = CatGlossaryUiPlugin.getDefault().getGlossaryEngine();
		final List<BentenGlossarySearchResult> entries = glossaryEngine.search(targetStrBuf.toString());

		if (CatGlossaryUiPlugin.IS_DEBUG) {
			final CatGlossaryUiPlugin plugin = CatGlossaryUiPlugin.getDefault();
			plugin.log(entries.size() + "件の用語がみつかりました。");
			for (int i = 0; i < entries.size(); i++) {
				final BentenGlossarySearchResult glossaryEntry = entries.get(i);
				plugin.log("------------------------------------"); //$NON-NLS-1$
				plugin.log((i + 1) + "件目");
				plugin.log(glossaryEntry.getSource());
				plugin.log(glossaryEntry.getTarget());
				plugin.log(glossaryEntry.getNote());
			}
			plugin.log("------------------------------------"); //$NON-NLS-1$
			plugin.log(""); //$NON-NLS-1$
		}

		viewer.setInput(entries);
	}
}
